/*
 *  Copyright (c) 2017 univento.eu - All rights reserved
 * You are not allowed to use, distribute or modify this code
 */

package eu.univento.core.api.items.attribute;

import lombok.AllArgsConstructor;
import net.minecraft.server.v1_12_R1.NBTTagCompound;

import java.util.UUID;

/**
 * @author joethei
 * @version 1.0
 */

@AllArgsConstructor
public class AttributeModifier {

    private Attribute attribute;
    private String name;
    private ItemSlot slot;
    private int operation;
    private double amount;
    private UUID uuid;

    /**
     * generates a new attribute modifier from an existing nbt tag
     * @param nbt NBTTagCompound of the item
     */
    public AttributeModifier(NBTTagCompound nbt) {
        this.attribute = Attribute.valueOf(nbt.getString("AttributeName"));
        this.name = nbt.getString("Name");
        this.slot = ItemSlot.valueOf("Slot");
        this.operation = nbt.getInt("Operation");
        this.amount = nbt.getDouble("Amount");
        this.uuid = new UUID(nbt.getLong("UUIDMost"), nbt.getLong("UUIDLeast"));
    }

    /**
     * generates a NBT Tag Compound
     * @return NBTTagCompound to be set on items
     */
    public NBTTagCompound getNBT() {
        NBTTagCompound nbt = new NBTTagCompound();
        nbt.setString("AttributeName", attribute.getName());
        nbt.setString("Name", name);
        nbt.setString("Slot", slot.getName());
        nbt.setInt("Operation", operation);
        nbt.setDouble("Amount", amount);
        nbt.setLong("UUIDMost", uuid.getMostSignificantBits());
        nbt.setLong("UUIDLeast", uuid.getLeastSignificantBits());

        return nbt;
    }
}